/*
Module     : pciedio.h
Version    : 1.0   
Date       : 04.07.2016 
Project    : Pciedio DLL
Environment: User mode DLL
  
THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
PURPOSE.
*/
#include <windows.h>

#ifndef PCIEDIOAPI_EXPORTS
  #ifdef __cplusplus
    #define PCIEDIOAPI extern "C" __declspec(dllimport)
  #else
    #define PCIEDIOAPI __declspec(dllimport)
  #endif
#endif

 
/*the PCI Express configuration data of a card*/
struct _PciedioConfig {  
  WORD    vendor;    
  WORD    device;  
  WORD    subvendor;      
  WORD    subsystem; 
  BYTE    revision; 
  BYTE    baseclass; 
  BYTE    subclass; 
  BYTE    headertype; 
  DWORD   address;  
  DWORD   range; 
  BYTE    type; 
  BYTE    interruptline;   
};

/*some standard values for the timer interval set up with PciedioSetTimerInterval*/
#define PCIEDIO_TIMER_10US        99UL
#define PCIEDIO_TIMER_50US       499UL
#define PCIEDIO_TIMER_100US      999UL
#define PCIEDIO_TIMER_500US     4999UL
#define PCIEDIO_TIMER_1MS       9999UL
#define PCIEDIO_TIMER_5MS      49999UL
#define PCIEDIO_TIMER_10MS     99999UL
#define PCIEDIO_TIMER_50MS    499999UL
#define PCIEDIO_TIMER_100MS   999999UL
#define PCIEDIO_TIMER_500MS  4999999UL
#define PCIEDIO_TIMER_1S     9999999UL
#define PCIEDIO_TIMER_MAX   0xFFFFFFUL   /*1.6777216s*/

/*some standard values for the FIFO interval set up with PciedioSetFifoInterval*/
#define PCIEDIO_FIFO_100US          9U
#define PCIEDIO_FIFO_500US         49U
#define PCIEDIO_FIFO_1MS           99U
#define PCIEDIO_FIFO_5MS          499U
#define PCIEDIO_FIFO_10MS         999U
#define PCIEDIO_FIFO_50MS        4999U
#define PCIEDIO_FIFO_100MS       9999U
#define PCIEDIO_FIFO_500MS      49999U
#define PCIEDIO_FIFO_MAX       0xFFFFU   /*0.65536s*/

/*some standard values for the watchdog interval set up with PciedioSetWatchdogInterval*/
#define PCIEDIO_WATCHDOG_100MS      4U   /*to be precise: 0.1048576s*/
#define PCIEDIO_WATCHDOG_500MS     20U   /*to be precise: 0.5242880s*/
#define PCIEDIO_WATCHDOG_1S        39U   /*to be precise: 1.0223616s*/
#define PCIEDIO_WATCHDOG_5S       191U   /*to be precise: 5.0069504s*/
#define PCIEDIO_WATCHDOG_MAX     0xFFU   /*6.684672s*/

#ifdef __cplusplus
extern "C" {
#endif  /* __cplusplus */

/*dll functions with BYTE:  1 byte unsigned 
                     WORD:  2 ytes unsigned
					 DWORD: 4 bytes unsigned 
					 Long:  4 bytes signed
*/

/*initialization*/
PCIEDIOAPI LONG __stdcall PciedioGetNumberOfCards(void);
PCIEDIOAPI LONG __stdcall PciedioOpenCards(void);
PCIEDIOAPI LONG __stdcall PciedioCloseCards(void);
PCIEDIOAPI LONG __stdcall PciedioGetIndices(BYTE *indices);
PCIEDIOAPI LONG __stdcall PciedioGetRevisions(BYTE cardnum, BYTE *revpcb, BYTE *revfpga, BYTE *revkernelhigh, BYTE *revkernellow, BYTE *revdllhigh, BYTE *revdlllow);
PCIEDIOAPI LONG __stdcall PciedioGetPciConfiguration(BYTE cardnum, struct _PciedioConfig *pciedioconfig);
PCIEDIOAPI LONG __stdcall PciedioGetAddressRange(BYTE cardnum, DWORD *address, DWORD *range, BYTE *type);
PCIEDIOAPI LONG __stdcall PciedioGetPresentJumperSettings(BYTE cardnum, BYTE *addressjumpers, BYTE *userjumper);
PCIEDIOAPI LONG __stdcall PciedioResetCard(BYTE cardnum);
PCIEDIOAPI LONG __stdcall PciedioOpenIrq(BYTE cardnum, void (__cdecl *hIsr)(BYTE cardnum, DWORD inputs, BYTE timer, BYTE fifo));
PCIEDIOAPI LONG __stdcall PciedioEnableGlobalIrq(BYTE cardnum);
PCIEDIOAPI LONG __stdcall PciedioDisableGlobalIrq(BYTE cardnum);


/*configuring i/os as input-only i/os*/
PCIEDIOAPI LONG __stdcall PciedioSetIOModeByte(BYTE cardnum, BYTE group, BYTE val);
PCIEDIOAPI LONG __stdcall PciedioSetIOModeWord(BYTE cardnum, BYTE group, WORD val);
PCIEDIOAPI LONG __stdcall PciedioSetIOModeDWord(BYTE cardnum, DWORD val);
PCIEDIOAPI LONG __stdcall PciedioSetIOModeBit(BYTE cardnum, BYTE channel, BYTE state);
PCIEDIOAPI LONG __stdcall PciedioGetIOModeByte(BYTE cardnum, BYTE group, BYTE *state);
PCIEDIOAPI LONG __stdcall PciedioGetIOModeWord(BYTE cardnum, BYTE group, WORD *state);
PCIEDIOAPI LONG __stdcall PciedioGetIOModeDWord(BYTE cardnum, DWORD *state);
PCIEDIOAPI LONG __stdcall PciedioGetIOModeBit(BYTE cardnum, BYTE channel, BYTE *state);


/*operating i/os as outputs*/
PCIEDIOAPI LONG __stdcall PciedioSetIOByte(BYTE cardnum, BYTE group, BYTE val);
PCIEDIOAPI LONG __stdcall PciedioSetIOWord(BYTE cardnum, BYTE group, WORD val);
PCIEDIOAPI LONG __stdcall PciedioSetIODWord(BYTE cardnum, DWORD val);
PCIEDIOAPI LONG __stdcall PciedioSetIOBitHigh(BYTE cardnum, BYTE channel);
PCIEDIOAPI LONG __stdcall PciedioSetIOBitLow(BYTE cardnum, BYTE channel);
PCIEDIOAPI LONG __stdcall PciedioSetIOBit(BYTE cardnum, BYTE channel, BYTE state);
PCIEDIOAPI LONG __stdcall PciedioSetIOUpdate(BYTE cardnum);
PCIEDIOAPI LONG __stdcall PciedioSetWatchdogInterval(BYTE cardnum, BYTE interval);
PCIEDIOAPI LONG __stdcall PciedioGetWatchdogSettings(BYTE cardnum, BYTE *interval, BYTE *state);
PCIEDIOAPI LONG __stdcall PciedioGetWatchdogState(BYTE cardnum, BYTE *state);


/*operating i/os as inputs*/
PCIEDIOAPI LONG __stdcall PciedioGetIOByte(BYTE cardnum, BYTE group, BYTE *state);
PCIEDIOAPI LONG __stdcall PciedioGetIOWord(BYTE cardnum, BYTE group, WORD *state);
PCIEDIOAPI LONG __stdcall PciedioGetIODWord(BYTE cardnum, DWORD *state);
PCIEDIOAPI LONG __stdcall PciedioGetIOBit(BYTE cardnum, BYTE channel, BYTE *state);

PCIEDIOAPI LONG __stdcall PciedioSetIOIrqEdgeByte(BYTE cardnum, BYTE group, BYTE edge);
PCIEDIOAPI LONG __stdcall PciedioSetIOIrqEdgeWord(BYTE cardnum, BYTE group, WORD edge);
PCIEDIOAPI LONG __stdcall PciedioSetIOIrqEdgeDWord(BYTE cardnum, DWORD edge);
PCIEDIOAPI LONG __stdcall PciedioSetIOIrqEdgeBit(BYTE cardnum, BYTE channel, BYTE edge);
PCIEDIOAPI LONG __stdcall PciedioGetIOIrqEdgeByte(BYTE cardnum, BYTE group, BYTE *state);
PCIEDIOAPI LONG __stdcall PciedioGetIOIrqEdgeWord(BYTE cardnum, BYTE group, WORD *state);
PCIEDIOAPI LONG __stdcall PciedioGetIOIrqEdgeDWord(BYTE cardnum, DWORD *state);
PCIEDIOAPI LONG __stdcall PciedioGetIOIrqEdgeBit(BYTE cardnum, BYTE channel, BYTE *state);

PCIEDIOAPI LONG __stdcall PciedioSetIOIrqEnableByte(BYTE cardnum, BYTE group, BYTE endis);
PCIEDIOAPI LONG __stdcall PciedioSetIOIrqEnableWord(BYTE cardnum, BYTE group, WORD endis);
PCIEDIOAPI LONG __stdcall PciedioSetIOIrqEnableDWord(BYTE cardnum, DWORD endis);
PCIEDIOAPI LONG __stdcall PciedioSetIOIrqEnableBit(BYTE cardnum, BYTE channel, BYTE endis);
PCIEDIOAPI LONG __stdcall PciedioGetIOIrqEnableByte(BYTE cardnum, BYTE group, BYTE *state);
PCIEDIOAPI LONG __stdcall PciedioGetIOIrqEnableWord(BYTE cardnum, BYTE group, WORD *state);
PCIEDIOAPI LONG __stdcall PciedioGetIOIrqEnableDWord(BYTE cardnum, DWORD *state);
PCIEDIOAPI LONG __stdcall PciedioGetIOIrqEnableBit(BYTE cardnum, BYTE channel, BYTE *state);

PCIEDIOAPI LONG __stdcall PciedioSetIOIrqConfigBit(BYTE cardnum, BYTE channel, BYTE endis, BYTE edge);
PCIEDIOAPI LONG __stdcall PciedioGetIOIrqConfigBit(BYTE cardnum, BYTE channel, BYTE *endis, BYTE *edge);

PCIEDIOAPI LONG __stdcall PciedioServiceIOIrqByte(BYTE cardnum, BYTE group, BYTE service);
PCIEDIOAPI LONG __stdcall PciedioServiceIOIrqWord(BYTE cardnum, BYTE group, WORD service);
PCIEDIOAPI LONG __stdcall PciedioServiceIOIrqDWord(BYTE cardnum, DWORD service);
PCIEDIOAPI LONG __stdcall PciedioServiceIOIrqBit(BYTE cardnum, BYTE channel);


/*operating the on-board FIFO*/
PCIEDIOAPI LONG __stdcall PciedioSetFifoInterval(BYTE cardnum, WORD interval);
PCIEDIOAPI LONG __stdcall PciedioStartFifo(BYTE cardnum);
PCIEDIOAPI LONG __stdcall PciedioStopFifo(BYTE cardnum);
PCIEDIOAPI LONG __stdcall PciedioClearFifo(BYTE cardnum);
PCIEDIOAPI LONG __stdcall PciedioGetFifoNumberOfEntries(BYTE cardnum, WORD *entries);
PCIEDIOAPI LONG __stdcall PciedioGetFifoEntries(BYTE cardnum, WORD number, DWORD *buffer);
PCIEDIOAPI LONG __stdcall PciedioSetFifoIrqLevel(BYTE cardnum, WORD entries);
PCIEDIOAPI LONG __stdcall PciedioEnableFifoIrq(BYTE cardnum);
PCIEDIOAPI LONG __stdcall PciedioDisableFifoIrq(BYTE cardnum);
PCIEDIOAPI LONG __stdcall PciedioServiceFifoIrq(BYTE cardnum);


/*operating the on-board timer*/
PCIEDIOAPI LONG __stdcall PciedioSetTimerInterval(BYTE cardnum, DWORD interval);
PCIEDIOAPI LONG __stdcall PciedioStartTimer(BYTE cardnum);
PCIEDIOAPI LONG __stdcall PciedioStopTimer(BYTE cardnum);
PCIEDIOAPI LONG __stdcall PciedioEnableTimerIrq(BYTE cardnum);
PCIEDIOAPI LONG __stdcall PciedioDisableTimerIrq(BYTE cardnum);
PCIEDIOAPI LONG __stdcall PciedioServiceTimerIrq(BYTE cardnum);
PCIEDIOAPI LONG __stdcall PciedioGetTimerPresentState(BYTE cardnum, DWORD *countval, BYTE *start, BYTE *irqen);

#ifdef __cplusplus
}
#endif
