/*
Module     : Fifo.cpp
Version    : 1.0   
Date       : 18.07.2016 
Project    : PCIeDIO sample
Environment: User mode
  
THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
PURPOSE.
*/

#include <stdio.h>
#include <conio.h>

#include "..\..\..\API\pciedio.h"

/**************************************************************************************************/
void __cdecl hIsr(BYTE cardnum, DWORD inputs, BYTE timer, BYTE fifo)
/**************************************************************************************************/
/*
Routine Description
    The interrupt service routine
*/
{ static DWORD error = 0;
  static DWORD run = 0;
  DWORD  buffer[20]; 
  LONG   retval; 
  DWORD  i;

  UNREFERENCED_PARAMETER(timer);
  UNREFERENCED_PARAMETER(inputs);

  if(fifo)
  { /********** get 20 entries of the FIFO while the FIFO continues collecting data **********/
    retval = PciedioGetFifoEntries(cardnum, 20, buffer);
    if(retval!=0)      
	 error++;    
	else
	{ printf("\n>> Run %08u: Show the next 20 entries (%u errors), hit any key to abort\n", run, error);
	  
	  for(i=0;i!=20;i++)  /*a simple printf*/
       printf("entry %02u: %08lxh\n",i,buffer[i]); 

	  run++;
	}

    /********** set up the next run **********/
    retval = PciedioServiceFifoIrq(cardnum);
	if(retval!=0)
 	 error++;
  }
  else
   error++; /*we don't expect to count interrupts from the timer or the IOs 
			  as we haven't enabled them
			*/
  return;
}


/**************************************************************************************************/
int main(void)
/**************************************************************************************************/
{ BYTE  cardnum;
  BYTE  indices;
  LONG  retval;
  WORD  entries;
  DWORD i;
  DWORD buffer[20];

  /********** open all the cards present in the system **********/
  retval = PciedioOpenCards();
  
  switch(retval)
  { case  0: printf(">> No PCIeDIO card present in the system\n");
             return 0;
             break;
    case  1: printf(">> 1 PCIeDIO card present in the system\n\n");
		     break;
	case  2:
	case  3:
	case  4:
	case  5:
	case  6:
	case  7:
	case  8:
		     printf(">> %d PCIeDIO cards present in the system\n\n",retval);
		 	 break;
	case -1:
	case -2: printf(">> PciedioOpenCards returned with error %d\n",retval);
		     return 0;
		 	 break;
    default: printf(">> PciedioOpenCards returned an impossible value %d\n", retval);
		     return 0;
			 break;
  }
  
  /********** determine the indices of the cards present in the system **********/
  PciedioGetIndices(&indices); 
  /*rem: no need to check the return value as we wouldn't be here when there was no card present*/
  
  /********** select the card with the lowest index present in the system *********/     
  for(cardnum=0;cardnum!=8;cardnum++)                    
  { if(indices >> cardnum & 0x01)           /*is a card present at the index?*/  
    { /*we have found the first valid index, so quit immediately*/           
      break;
    }
  }  
 
  printf(">> The card at index %u will be used to demonstrate the usage of the FIFO\n", cardnum);
 


  /***********************/
  /*non-interrupted usage*/
  /***********************/

  /********* set the interval of the FIFO timer to 100ms **********/
  retval = PciedioSetFifoInterval(cardnum,PCIEDIO_FIFO_100MS);
  if(retval!=0)
  { printf(">> PciedioSetFifoInterval returned with error %d\n",retval);
    goto EXIT;
  }
  
  /********* start the FIFO timer **********/
  retval = PciedioStartFifo(cardnum);
  if(retval!=0)
  { printf(">> PciedioStartFifo returned with error %d\n",retval);
    goto EXIT;
  }
 
  while(!_kbhit())
  { static DWORD run = 0;

	printf("\n>> Run %08u: Wait on 20 entries, hit any key to abort\n",run);

    do
    { /********** get the number of entries currently stored in the FIFO **********/
	  retval = PciedioGetFifoNumberOfEntries(cardnum, &entries);
      if(retval!=0)
      { printf(">> PciedioGetFifoNumberOfEntries returned with error %d\n",retval);
        goto EXIT;
      }
    }
    while(entries < 20);
    
    
    /********** get 20 entries of the FIFO while the FIFO continues collecting **********/
    retval = PciedioGetFifoEntries(cardnum, 20, buffer);
    if(retval!=0)
    { printf(">> PciedioGetFifoEntries returned with error %d\n",retval);
      goto EXIT;
    } 

    for(i=0;i!=20;i++)  /*a simple printf*/
     printf("entry %02u: %08lxh\n",i,buffer[i]);

	run++;
  } 
  _getch();

  /********* stop the FIFO from collecting data **********/
  retval = PciedioStopFifo(cardnum);
  if(retval!=0)
  { printf(">> PciedioStopFifo returned with error %d\n",retval);
    goto EXIT;
  }

  /********* clear potential remaining contents for the next demonstration **********/
  retval = PciedioClearFifo(cardnum);
  if(retval!=0)
  { printf(">> PciedioClearFifo returned with error %d\n",retval);
    goto EXIT;
  }


  /*****************************************************/
  /*interrupted usage: we use the same settings a above*/
  /*****************************************************/

  /********* set the interval of the FIFO timer to 100ms **********/
  retval = PciedioSetFifoInterval(cardnum,PCIEDIO_FIFO_100MS);
  if(retval!=0)
  { printf(">> PciedioSetFifoInterval returned with error %d\n",retval);
    goto EXIT;
  }

  /********** install the isr **********/
  retval = PciedioOpenIrq(cardnum, &hIsr);
  if(retval!=0)
  { printf(">> PciedioOpenIrq returned with error %d\n",retval);
    goto EXIT;
  }

  /********* set the alarm level of the FIFO to 20 entries **********/
  retval = PciedioSetFifoIrqLevel(cardnum,20); 
  if(retval!=0)
  { printf(">> PciedioSetFifoIrqLevel returned with error %d\n",retval);
    goto EXIT;
  }

  /********* enable the FIFO as an interrupt source **********/
  retval = PciedioEnableFifoIrq(cardnum); 
  if(retval!=0)
  { printf(">> PciedioEnableFifoIrq returned with error %d\n",retval);
    goto EXIT;
  }

  /********* enable interrupts globally **********/
  retval = PciedioEnableGlobalIrq(cardnum);
  if(retval!=0)
  { printf(">> PciedioEnableGlobalIrq returned with error %d\n",retval);
    goto EXIT;
  }

  /********* start the FIFO **********/
  retval = PciedioStartFifo(cardnum);
  if(retval!=0)
  { printf(">> PciedioStartFifo returned with error %d\n",retval);
    goto EXIT;
  }

  /********* let the interrupt service routine carry out the work **********/
  while(_kbhit())
   ;
  _getch();
  
  /********* stop the FIFO from collecting data **********/
  retval = PciedioStopFifo(cardnum);
  if(retval!=0)
  { printf(">> PciedioStopFifo returned with error %d\n",retval);
    goto EXIT;
  }

  /********** disable the irq globally **********/
  retval = PciedioDisableGlobalIrq(cardnum);
  if(retval!=0)
  { printf(">> PciedioDisableGlobalIrq returned with error %d\n",retval);
    goto EXIT;
  } 

  /********** disable the irq locally **********/
  retval = PciedioDisableFifoIrq(cardnum);
  if(retval!=0)  
  { printf(">> PciedioDisableFifoIrq returned with error %d\n",retval);
    goto EXIT;
  } 


  
EXIT:
  /********** reset the card to its default state **********/
  retval = PciedioResetCard(cardnum);
  if(retval!=0)  
   printf(">> PciedioReset returned with error %d\n",retval);
  
  /********** release any resources allocated by PciedioOpenCards **********/ 
  retval = PciedioCloseCards();
  if(retval!=0)
   printf(">> PciedioCloseCards returned with error %d\n",retval);

  printf("\n\nHit any key to quit the program\n");
  
  _getch(); 

  return 0;
}
