/*
Module     : IOBasicOperation.cpp
Version    : 1.0   
Date       : 18.07.2016 
Project    : PCIeDIO sample
Environment: User mode
  
THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
PURPOSE.
*/

#include <stdio.h>
#include <conio.h>

#include "..\..\..\API\pciedio.h"

void delay_ms(WORD wait); /*perform a simple delay using the Windows API*/
DWORD random(void);       /*build a random 32-bit pattern using the Windows API*/

/**************************************************************************************************/
int main(void)
/**************************************************************************************************/
{ BYTE  cardnum;
  BYTE  indices;
  LONG  retval;

  union { BYTE  byte;
	      WORD  word;
	      DWORD dword;
	    } input;

  INT   c;

  /********** open all the cards present in the system **********/
  retval = PciedioOpenCards();
  
  switch(retval)
  { case  0: printf(">> No PCIeDIO card present in the system\n");
             return 0;
             break;
    case  1: printf(">> 1 PCIeDIO card present in the system\n\n");
		     break;
	case  2:
	case  3:
	case  4:
	case  5:
	case  6:
	case  7:
	case  8:
		     printf(">> %d PCIeDIO cards present in the system\n\n",retval);
		 	 break;
	case -1:
	case -2: printf(">> PciedioOpenCards returned with error %d\n",retval);
		     return 0;
		 	 break;
    default: printf(">> PciedioOpenCards returned an impossible value %d\n", retval);
		     return 0;
			 break;
  }
  
  /********** determine the indices of the cards present in the system **********/
  PciedioGetIndices(&indices); 
  /*rem: no need to check the return value as we wouldn't be here when there was no card present*/
  
  /********** select the card with the lowest index present in the system *********/     
  for(cardnum=0;cardnum!=8;cardnum++)                    
  { if(indices >> cardnum & 0x01)           /*is a card present at the index?*/  
    { /*we have found the first valid index, so quit immediately*/           
      break;
    }
  }  
  
  printf(">> The card at index %u will be used to demonstrate the basic usage of the IOs\n\n",cardnum);
  
  printf(">> WARNING!\n");
  printf("   Before running this application, disconnect all IOs to prevent any damage\n");
  printf("   as the application uses all IOs as outputs to stimulate the IOs as inputs\n"); 
  printf("   Keep the external power connected to the card to operate the sample\n\n");
  printf(">> Press y to proceed or abort with q\n\n");
  
  c = 'a';
  while(c!='y')
  { if(_kbhit())
     { c = _getch();
       if(c=='q')
        goto EXIT;
     }
  }

  printf("Application is running, press any key to abort\n");

  while(!_kbhit())
  { DWORD outpattern;
    INT i;
    
	/*********************/
	/*using DWORD access*/
    /********************/
    outpattern = random();

	/********** set the output pattern using DWORD access**********/
	retval = PciedioSetIODWord(cardnum, outpattern);
    if(retval!=0)  
     printf(">> PciedioSetIODWord returned with error %d\n",retval);
    
	/********** wait 1ms **********/
	delay_ms(1);

	/********** get the input pattern using DWORD access **********/
	retval = PciedioGetIODWord(cardnum, &input.dword);
	if(retval!=0)  
     printf(">> PciedioGetIODWord returned with error %d\n",retval);
	else
	{ if(input.dword != outpattern)
       printf(">> ERROR: input pattern %08lxh does not match the output pattern %08lxh\n", input.dword, outpattern);
	}
   
    /*********************/
	/*using WORD access*/
    /********************/
    outpattern = random();

	/********** set the output pattern using WORD access**********/
	retval = PciedioSetIOWord(cardnum, 0, (WORD) outpattern);         /*first I/O group of 16 IOs*/
    if(retval!=0)  
     printf(">> PciedioSetIOWord returned with error %d\n",retval);
    
	retval = PciedioSetIOWord(cardnum, 1, (WORD) (outpattern >> 16)); /*second I/O group of 16 IOs*/
    if(retval!=0)  
     printf(">> PciedioSetIOWord returned with error %d\n",retval);

	/********** wait 1ms **********/
	delay_ms(1);

	/********** get the input pattern using WORD access ***********/
	retval = PciedioGetIOWord(cardnum, 0, &input.word);               /*first I/O group of 16 IOs*/
	if(retval!=0)  
     printf(">> PciedioGetIOWord returned with error %d\n",retval);
	else
	{ if(input.word != (WORD) outpattern)
       printf(">> ERROR: input pattern %04xh does not match the output pattern %04xh\n", input.word, (WORD) outpattern);
	}

	retval = PciedioGetIOWord(cardnum, 1, &input.word);               /*second I/O group of 16 IOs*/
	if(retval!=0)  
     printf(">> PciedioGetIOWord returned with error %d\n",retval);
	else
	{ if(input.word != (WORD) (outpattern >> 16))
       printf(">> ERROR: input pattern %04xh does not match the output pattern %04xh\n", input.word, (WORD) (outpattern >> 16));
	}

	/*********************/
	/*using BYTE access*/
    /********************/
    outpattern = random();

	/********** set the output pattern using BYTE access **********/
	retval = PciedioSetIOByte(cardnum, 0, (BYTE) outpattern);         /*first I/O group of 8 IOs*/
    if(retval!=0)  
     printf(">> PciedioSetIOByte returned with error %d\n",retval);   
    
	retval = PciedioSetIOByte(cardnum, 1, (BYTE) (outpattern >> 8));  /*second I/O group of 8 IOs*/
    if(retval!=0)  
     printf(">> PciedioSetIOByte returned with error %d\n",retval);
	
	retval = PciedioSetIOByte(cardnum, 2, (BYTE) (outpattern >> 16)); /*third I/O group of 8 IOs*/ 
    if(retval!=0)  
     printf(">> PciedioSetIOByte returned with error %d\n",retval);
	
	retval = PciedioSetIOByte(cardnum, 3, (BYTE) (outpattern >> 24)); /*fourth I/O group of 8 IOs*/
    if(retval!=0)  
     printf(">> PciedioSetIOByte returned with error %d\n",retval);

	/********** wait 1ms **********/
	delay_ms(1);

	/********** get the input pattern using BYTE access **********/
	retval = PciedioGetIOByte(cardnum, 0, &input.byte);               /*first I/O group of 8 IOs*/ 
	if(retval!=0)  
     printf(">> PciedioGetIOByte returned with error %d\n",retval);
	else
	{ if(input.byte != (BYTE) outpattern)
       printf(">> ERROR: input pattern %02xh does not match the output pattern %02xh\n", input.byte, (BYTE) outpattern);
	}

	retval = PciedioGetIOByte(cardnum, 1, &input.byte);               /*second I/O group of 8 IOs*/
	if(retval!=0)  
     printf(">> PciedioGetIOByte returned with error %d\n",retval);
	else
	{ if(input.byte != (BYTE) (outpattern >> 8))
       printf(">> ERROR: input pattern %02xh does not match the output pattern %02xh\n", input.byte, (BYTE) (outpattern >> 8));
	}
	
	retval = PciedioGetIOByte(cardnum, 2, &input.byte);               /*third I/O group of 8 IOs*/
	if(retval!=0)  
     printf(">> PciedioGetIOByte returned with error %d\n",retval);
	else
	{ if(input.byte != (BYTE) (outpattern >> 16))
       printf(">> ERROR: input pattern %02xh does not match the output pattern %02xh\n", input.byte, (BYTE) (outpattern >> 16));
	}
	
	retval = PciedioGetIOByte(cardnum, 3, &input.byte);               /*fourth I/O group of 8 IOs*/
	if(retval!=0)  
     printf(">> PciedioGetIOByte returned with error %d\n",retval);
	else
	{ if(input.byte != (BYTE) (outpattern >> 24))
       printf(">> ERROR: input pattern %02xh does not match the output pattern %02xh\n", input.byte, (BYTE) (outpattern >> 24));
	}

   
	/****************************/
	/*using BIT access method 1 */
    /****************************/
	outpattern = random();

	/********** set the output pattern using BIT access method one **********/
    for(i=0;i!=32;i++)
	{ retval = PciedioSetIOBit(cardnum, (BYTE) i, (BYTE) (outpattern >> i & 0x01));
      if(retval!=0)  
       printf(">> PciedioSetIOBit returned with error %d\n",retval);
	}
    
	/********** wait 1ms **********/
	delay_ms(1);

	/********** get the input pattern using BIT access **********/
	for(i=0;i!=32;i++)
	{ retval = PciedioGetIOBit(cardnum, (BYTE) i, &input.byte);
      if(retval!=0)  
       printf(">> PciedioGetIOBit returned with error %d\n",retval);
	  else
	  { if(input.byte != (BYTE) (outpattern >> i & 0x01))
         printf(">> ERROR: input level does not match the output level at channel IO%02u\n", i); 
   	  }
	}

	/****************************/
	/*using BIT access method 2 */
    /****************************/
    outpattern = random();

	/********** set the output pattern using BIT access method two **********/
    for(i=0;i!=32;i++)
	{ if((outpattern >> i & 0x01) == 1)
	   retval = PciedioSetIOBitHigh(cardnum, (BYTE) i);
	  else
       retval = PciedioSetIOBitLow(cardnum, (BYTE) i);
      if(retval!=0)  
       printf(">> PciedioSetIOBitHigh/Low returned with error %d\n",retval);
	}
    
	/********** wait 1ms **********/
	delay_ms(1);

	/********** get the input pattern using BIT access **********/
	for(i=0;i!=32;i++)
	{ retval = PciedioGetIOBit(cardnum, (BYTE) i, &input.byte);
      if(retval!=0)  
       printf(">> PciedioGetIOBit returned with error %d\n",retval);
	  else
	  { if(input.byte != (BYTE) (outpattern >> i & 0x01))
         printf(">> ERROR: input level does not match the output level at channel IO%02u\n", i); 
   	  }
	}

  }
  _getch();
  

EXIT:
  /********** reset the card to its default state **********/
  retval = PciedioResetCard(cardnum);
  if(retval!=0)  
   printf(">> PciedioReset returned with error %d\n",retval);
  
  /********** release any resources allocated by PciedioOpenCards **********/ 
  retval = PciedioCloseCards();
  if(retval!=0)
   printf(">> PciedioCloseCards returned with error %d\n",retval);

  printf("\n\nHit any key to quit the program\n");
  
  _getch(); 

  return 0;
}



/**************************************************************************************************/
void delay_ms(WORD wait)
/**************************************************************************************************/
/*
Routine Description
    Perform a simple delay using the Windows API
*/
{ LARGE_INTEGER frequency; 
  LARGE_INTEGER t1, t2; 

  QueryPerformanceFrequency(&frequency);
 
  QueryPerformanceCounter(&t1);
  
  do
   QueryPerformanceCounter(&t2);
  while((1000.0*(t2.QuadPart-t1.QuadPart)/frequency.QuadPart) < wait);

  return;
}


/**************************************************************************************************/
DWORD random(void)
/**************************************************************************************************/
/*
Routine Description
    Build a random 32-bit pattern using the Windows API
*/
{ DWORD pattern;
  
  pattern = (DWORD) rand() << 17; 

  pattern |= (DWORD) rand() << 2;
  
  pattern |= (DWORD) rand() & 0x03;  
  
  return pattern;
}


