/*
Module     : IOSetModeConfiguration.cpp
Version    : 1.0   
Date       : 18.07.2016 
Project    : PCIeDIO sample
Environment: User mode
  
THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
PURPOSE.
*/

#include <stdio.h>
#include <conio.h>

#include "..\..\..\API\pciedio.h"

DWORD random(void);       /*build a random 32-bit pattern using the Windows API*/

/**************************************************************************************************/
int main(void)
/**************************************************************************************************/
{ BYTE  cardnum;
  BYTE  indices;
  LONG  retval;
  INT   i;
  DWORD modepattern;
  
  union { BYTE  byte;
	      WORD  word;
	      DWORD dword;
	    } input;

  /********** open all the cards present in the system **********/
  retval = PciedioOpenCards();
  
  switch(retval)
  { case  0: printf(">> No PCIeDIO card present in the system\n");
             return 0;
             break;
    case  1: printf(">> 1 PCIeDIO card present in the system\n\n");
		     break;
	case  2:
	case  3:
	case  4:
	case  5:
	case  6:
	case  7:
	case  8:
		     printf(">> %d PCIeDIO cards present in the system\n\n",retval);
		 	 break;
	case -1:
	case -2: printf(">> PciedioOpenCards returned with error %d\n",retval);
		     return 0;
		 	 break;
    default: printf(">> PciedioOpenCards returned an impossible value %d\n", retval);
		     return 0;
			 break;
  }
  
  /********** determine the indices of the cards present in the system **********/
  PciedioGetIndices(&indices); 
  /*rem: no need to check the return value as we wouldn't be here when there was no card present*/
  
  /********** select the card with the lowest index present in the system *********/     
  for(cardnum=0;cardnum!=8;cardnum++)                    
  { if(indices >> cardnum & 0x01)           /*is a card present at the index?*/  
    { /*we have found the first valid index, so quit immediately*/           
      break;
    }
  }  
  
  printf(">> The card at index %u will be used to demonstrate how to set the configuration\n\n",cardnum);
  
   /*********************/
  /*using DWORD access*/
  /********************/
  modepattern = random();
  
  /********** set the mode pattern using DWORD access**********/
  retval = PciedioSetIOModeDWord(cardnum, modepattern);
  if(retval!=0)  
   printf(">> PciedioSetIOModeDWord returned with error %d\n",retval);
    
  /********** verify mode pattern using DWORD access **********/
  retval = PciedioGetIOModeDWord(cardnum, &input.dword);
  if(retval!=0)  
   printf(">> PciedioGetIOModeDWord returned with error %d\n",retval);
  else
   { if(input.dword != modepattern)
      printf(">> ERROR: returned mode pattern %08lxh does not match the set mode pattern %08lxh\n", input.dword, modepattern);
	 else
	 { printf(">> The following configuration has been set up using mode pattern %08lxh:\n",modepattern);
       for(i=0;i!=32;i++)
	   { if(input.dword >> i & 0x01)
	      printf("   IO%02u can be used as an output\n",i);
	     else
          printf("   IO%02u is configured as input-only\n",i);
	   }    
	 }
   }
  printf("\n");


  /*********************/
  /*using WORD access*/
  /********************/
  modepattern = random();
  
  /********** set the mode pattern using WORD access**********/
  retval = PciedioSetIOModeWord(cardnum, 0, (WORD) modepattern);           /*first I/O group of 16 IOs*/
  if(retval!=0)  
   printf(">> PciedioSetIOModeWord returned with error %d\n",retval);
    
  /********** verify mode pattern using WORD access **********/
  retval = PciedioGetIOModeWord(cardnum, 0, &input.word);                  /*first I/O group of 16 IOs*/
  if(retval!=0)  
   printf(">> PciedioGetIOModeWord returned with error %d\n",retval);
  else
   { if(input.word != (WORD) modepattern)
      printf(">> ERROR: returned mode pattern %04xh does not match the set mode pattern %04xh\n", input.word, (WORD) modepattern);
	 else
	 { printf(">> The following configuration has been set up using mode pattern %04xh:\n",(WORD) modepattern);
       for(i=0;i!=16;i++)
	   { if(input.dword >> i & 0x01)
	      printf("   IO%02u can be used as an output\n",i);
	     else
          printf("   IO%02u is configured as input-only\n",i);
	   }    
	 }
   }
   
  /********** set the mode pattern using WORD access**********/
  retval = PciedioSetIOModeWord(cardnum, 1, (WORD) (modepattern >> 16));   /*second I/O group of 16 IOs*/
  if(retval!=0)  
   printf(">> PciedioSetIOModeWord returned with error %d\n",retval);
    
  /********** verify mode pattern using WORD access **********/
  retval = PciedioGetIOModeWord(cardnum, 1, &input.word);                  /*second I/O group of 16 IOs*/
  if(retval!=0)  
   printf(">> PciedioGetIOModeWord returned with error %d\n",retval);
  else
   { if(input.word != (WORD) (modepattern >> 16))
      printf(">> ERROR: returned mode pattern %04xh does not match the set mode pattern %04xh\n", 
	                                                         input.word, (WORD) (modepattern >> 16)
	  );
	 else
	 { printf(">> The following configuration has been set up using mode pattern %04xh:\n",
	                                                              (WORD) (modepattern >> 16)
       );

       for(i=0;i!=16;i++)
	   { if(input.dword >> i & 0x01)
	      printf("   IO%02u can be used as an output\n",i+16);
	     else
          printf("   IO%02u is configured as input-only\n",i+16);
	   }    
	 }
   }
  printf("\n"); 


  /*********************/
  /*using BYTE access*/
  /********************/
  modepattern = random();
  
  /********** set the mode pattern using BYTE access**********/
  retval = PciedioSetIOModeByte(cardnum, 0, (BYTE) modepattern);           /*first I/O group of 8 IOs*/
  if(retval!=0)  
   printf(">> PciedioSetIOModeByte returned with error %d\n",retval);
    
  /********** verify mode pattern using BYTE access **********/
  retval = PciedioGetIOModeByte(cardnum, 0, &input.byte);                  /*first I/O group of 8 IOs*/
  if(retval!=0)  
   printf(">> PciedioGetIOModeByte returned with error %d\n",retval);
  else
   { if(input.byte != (BYTE) modepattern)
      printf(">> ERROR: returned mode pattern %02xh does not match the set mode pattern %02xh\n", input.byte, (BYTE) modepattern);
	 else
	 { printf(">> The following configuration has been set up using mode pattern %02xh:\n",(BYTE) modepattern);
       for(i=0;i!=8;i++)
	   { if(input.byte >> i & 0x01)
	      printf("   IO%02u can be used as an output\n",i);
	     else
          printf("   IO%02u is configured as input-only\n",i);
	   }    
	 }
   }

  /********** set the mode pattern using BYTE access**********/
  retval = PciedioSetIOModeByte(cardnum, 1, (BYTE) (modepattern >> 8));    /*second I/O group of 8 IOs*/
  if(retval!=0)  
   printf(">> PciedioSetIOModeByte returned with error %d\n",retval);
    
  /********** verify mode pattern using BYTE access **********/
  retval = PciedioGetIOModeByte(cardnum, 1, &input.byte);                  /*second I/O group of 8 IOs*/ 
  if(retval!=0)  
   printf(">> PciedioGetIOModeByte returned with error %d\n",retval);
  else
   { if(input.byte != (BYTE) (modepattern >> 8))
      printf(">> ERROR: returned mode pattern %02xh does not match the set mode pattern %02xh\n", input.byte, (BYTE) (modepattern >> 8));
	 else
	 { printf(">> The following configuration has been set up using mode pattern %02xh:\n",(BYTE) (modepattern >> 8));
       for(i=0;i!=8;i++)
	   { if(input.byte >> i & 0x01)
	      printf("   IO%02u can be used as an output\n",i+8);
	     else
          printf("   IO%02u is configured as input-only\n",i+8);
	   }    
	 }
   }

  /********** set the mode pattern using BYTE access**********/
  retval = PciedioSetIOModeByte(cardnum, 2, (BYTE) (modepattern >> 16));   /*third I/O group of 8 IOs*/
  if(retval!=0)  
   printf(">> PciedioSetIOModeByte returned with error %d\n",retval);
    
  /********** verify mode pattern using BYTE access **********/
  retval = PciedioGetIOModeByte(cardnum, 2, &input.byte);                  /*third I/O group of 8 IOs*/
  if(retval!=0)  
   printf(">> PciedioGetIOModeByte returned with error %d\n",retval);
  else
   { if(input.byte != (BYTE) (modepattern >> 16))
      printf(">> ERROR: returned mode pattern %02xh does not match the set mode pattern %02xh\n", input.byte, (BYTE) (modepattern >> 16));
	 else
	 { printf(">> The following configuration has been set up using mode pattern %02xh:\n",(BYTE) (modepattern >> 16));
       for(i=0;i!=8;i++)
	   { if(input.byte >> i & 0x01)
	      printf("   IO%02u can be used as an output\n",i+16);
	     else
          printf("   IO%02u is configured as input-only\n",i+16);
	   }    
	 }
   }

  /********** set the mode pattern using BYTE access**********/
  retval = PciedioSetIOModeByte(cardnum, 3, (BYTE) (modepattern >> 24));   /*fourth I/O group of 8 IOs*/
  if(retval!=0)  
   printf(">> PciedioSetIOModeByte returned with error %d\n",retval);
    
  /********** verify mode pattern using BYTE access **********/
  retval = PciedioGetIOModeByte(cardnum, 3, &input.byte);                  /*fourth I/O group of 8 IOs*/
  if(retval!=0)  
   printf(">> PciedioGetIOModeByte returned with error %d\n",retval);
  else
   { if(input.byte != (BYTE) (modepattern >> 24))
      printf(">> ERROR: returned mode pattern %02xh does not match the set mode pattern %02xh\n", input.byte, (BYTE) (modepattern >> 24));
	 else
	 { printf(">> The following configuration has been set up using mode pattern %02xh:\n",(BYTE) (modepattern >> 24));
       for(i=0;i!=8;i++)
	   { if(input.byte >> i & 0x01)
	      printf("   IO%02u can be used as an output\n",i+24);
	     else
          printf("   IO%02u is configured as input-only\n",i+24);
	   }    
	 }
   }
  printf("\n");

  /*********************/
  /*using BIT access*/
  /********************/
  modepattern = random();
  
  /********** set the mode pattern using BIT access**********/
  for(i=0;i!=32;i++)
  { if(modepattern >> i & 0x01)
     retval = PciedioSetIOModeBit(cardnum, (BYTE) i, 1);     /*the IO can be uesed as an output*/
    else
     retval = PciedioSetIOModeBit(cardnum, (BYTE) i, 0);     /*the IO is input-only*/
  
    if(retval!=0)  
     printf(">> PciedioSetIOModeBit returned with error %d\n",retval);
  }

  printf(">> The following configuration has been set up using mode pattern %08lxh:\n",modepattern);
  /********** get the mode pattern using BIT access**********/
  for(i=0;i!=32;i++)
  { retval = PciedioGetIOModeBit(cardnum, (BYTE) i, &input.byte);
    if(retval!=0)  
     printf(">> PciedioGetIOModeBit returned with error %d\n",retval);
	
	if(input.byte != (BYTE) (modepattern >> i & 0x01))
     printf(">> ERROR: returned mode does not match the set mode at channel IO%02u\n", i); 
	else
	{ if(modepattern >> i & 0x01)
	   printf("   IO%02u can be used as an output\n",i);
	  else
       printf("   IO%02u is configured as input-only\n",i);
	}
  }


  /********** reset the card to its default state **********/
  retval = PciedioResetCard(cardnum);
  if(retval!=0)  
   printf(">> PciedioReset returned with error %d\n",retval);
  
  /********** release any resources allocated by PciedioOpenCards **********/ 
  retval = PciedioCloseCards();
  if(retval!=0)
   printf(">> PciedioCloseCards returned with error %d\n",retval);

  printf("\n\nHit any key to quit the program\n");
  
  _getch(); 

  return 0;
}


/**************************************************************************************************/
DWORD random(void)
/**************************************************************************************************/
/*
Routine Description
    Build a random 32-bit pattern using the Windows API
*/
{ DWORD pattern;
  
  pattern = (DWORD) rand() << 17; 

  pattern |= (DWORD) rand() << 2;
  
  pattern |= (DWORD) rand() & 0x03;  
  
  return pattern;
}


