/*
Module     : InterruptHandling.cpp
Version    : 1.0   
Date       : 18.07.2016 
Project    : PCIeDIO sample
Environment: User mode
  
THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
PURPOSE.
*/

#include <stdio.h>
#include <conio.h>


#include "..\..\..\API\pciedio.h"

void delay_ms(WORD wait); /*perform a simple delay using the Windows API*/

DWORD buffer[100];        /*the buffer for the FIFO*/

/**************************************************************************************************/
void __cdecl hIsr(BYTE cardnum, DWORD inputs, BYTE timer, BYTE fifo)
/**************************************************************************************************/
/*
Routine Description
    The interrupt service routine
*/
{ static DWORD timercount = 0;
  static DWORD fifocount = 0;
  static DWORD iochannel00count = 0;
  
  static DWORD error = 0;
 
  LONG   retval; 
  BYTE   iochannel00;

  if(inputs) /*has any IO channel triggered an interrupt?*/
  { if(inputs & 0x01)  /*has IO00 triggered an interrupt?*/
     { /********** service IO00 interrupt **********/
	   retval = PciedioServiceIOIrqBit(cardnum,
		                                     0); /*service channel IO00*/
	   if(retval!=0)
 	     error++;        

	   iochannel00count++;
     }
	else
     error++; /*we don't expect to count interrupts from other inputs 
			   as we haven't enabled them
			  */
  }

  if(fifo)
  { /*in main(), the FIFO timer interval is set to 2.5ms and the alarm level to 100 so that 
	  every 250ms the FIFO interrupt triggers*/
	
	/********** get 100 entries of the FIFO while the FIFO continues collecting data **********/    
	retval = PciedioGetFifoEntries(cardnum, 100, buffer);
    if(retval!=0)      
	 error++;    	

    /********** set up the next run **********/
    retval = PciedioServiceFifoIrq(cardnum);
	if(retval!=0)
 	 error++;

	fifocount++;
  }

  if(timer)
  { /********** read present state of IO00 to variable iochannel00 **********/
	retval = PciedioGetIOBit(cardnum,
		                           0,      /*channel IO00*/
					    &iochannel00);
    if(retval!=0)
 	 error++;

	/*toggle the state of IO00 and write IO00*/
	if(iochannel00==0)
	 iochannel00 = 1;
	else
	 iochannel00 = 0;
    retval = PciedioSetIOBit(cardnum,
		                           0,      /*channel IO00*/
					     iochannel00);
    if(retval!=0)
 	 error++;
     
    /********** set up the next run **********/
    retval = PciedioServiceTimerIrq(cardnum);
	if(retval!=0)
 	 error++;
	
    /*a simple printf to show the present state of the variables every 200 timer interrupts = 1s*/
	/*rem: a printf in an interrupt routine may delay the processing of the routine and the start
	  of the next run*/
	if(timercount%200==0)
     printf("card %u interrupts: %u timer, %u fifo and %u IO00 with %u error(s)\r",
            cardnum, timercount, fifocount, iochannel00count, error);

	timercount++;
  }
 
  return;
}


/**************************************************************************************************/
int main(void)
/**************************************************************************************************/
{ BYTE  cardnum;
  BYTE  indices;
  LONG  retval;
  INT   i;
  INT   c;
  BYTE  iochannel04; 

  /********** open all the cards present in the system **********/
  retval = PciedioOpenCards();
  
  switch(retval)
  { case  0: printf(">> No PCIeDIO card present in the system\n");
             return 0;
             break;
    case  1: printf(">> 1 PCIeDIO card present in the system\n\n");
		     break;
	case  2:
	case  3:
	case  4:
	case  5:
	case  6:
	case  7:
	case  8:
		     printf(">> %d PCIeDIO cards present in the system\n\n",retval);
		 	 break;
	case -1:
	case -2: printf(">> PciedioOpenCards returned with error %d\n",retval);
		     return 0;
		 	 break;
    default: printf(">> PciedioOpenCards returned an impossible value %d\n", retval);
		     return 0;
			 break;
  }
  
  /********** determine the indices of the cards present in the system **********/
  PciedioGetIndices(&indices); 
  /*rem: no need to check the return value as we wouldn't be here when there was no card present*/
  
  /********** select the card with the lowest index present in the system *********/     
  for(cardnum=0;cardnum!=8;cardnum++)                    
  { if(indices >> cardnum & 0x01)           /*is a card present at the index?*/  
    { /*we have found the first valid index, so quit immediately*/           
      break;
    }
  }  
  
  printf(">> The card at index %u will be used to demonstrate the interrupt handling\n",cardnum);
  
  printf("\n>> WARNING!\n");
  printf("   Before running this application, disconnect all IOs to prevent any damage\n");
  printf("   as the application uses IO00 and IO04 as outputs\n"); 
  printf("   Keep the external power connected to the card to operate the sample\n\n");
  printf(">> Press y to proceed or abort with q\n\n");
  
  c = 'a';
  while(c!='y')
  { if(_kbhit())
     { c = _getch();
       if(c=='q')
        goto EXIT;
     }
  }
  
  /********** install the isr **********/
  retval = PciedioOpenIrq(cardnum, &hIsr);
  if(retval!=0)
  { printf(">> PciedioOpenIrq returned with error %d\n",retval);
    goto EXIT;
  }

  /********* set up the timer interval to 5ms to toggle IO00 **********/
  retval = PciedioSetTimerInterval(cardnum,PCIEDIO_TIMER_5MS);
  if(retval!=0)
  { printf(">> PciedioSetTimerInterval returned with error %d\n",retval);
    goto EXIT;
  }

  /********* enable the timer as an interrupt source **********/
  retval = PciedioEnableTimerIrq(cardnum);
  if(retval!=0)
  { printf(">> PciedioEnableTimerIrq returned with error %d\n",retval);
    goto EXIT;
  }
  
  /********* set the interval of the FIFO timer to 2.5ms **********/
  retval = PciedioSetFifoInterval(cardnum,249); /*249 -> cycletime = (249 + 1) * 10us = 2.5ms*/
  if(retval!=0)
  { printf(">> PciedioSetFifoInterval returned with error %d\n",retval);
    goto EXIT;
  }

  /********* set the alarm level of the FIFO to 100 entries **********/
  /* -> receive an FIFO interrupt every 250ms as the FIFO timer interval was set to 2.5ms*/
  retval = PciedioSetFifoIrqLevel(cardnum,100); 
  if(retval!=0)
  { printf(">> PciedioSetFifoIrqLevel returned with error %d\n",retval);
    goto EXIT;
  }

  /********* enable the FIFO as an interrupt source **********/
  retval = PciedioEnableFifoIrq(cardnum); 
  if(retval!=0)
  { printf(">> PciedioEnableFifoIrq returned with error %d\n",retval);
    goto EXIT;
  }

  /********* set the IO00 interrupt triggering edge from low to high **********/
  retval = PciedioSetIOIrqEdgeBit(cardnum,
	                                    0,     /*channel IO00*/
	                                    1);    /*edge = 1: low to high triggers the interrupt*/
		     
  if(retval!=0)
  { printf(">> PciedioSetIOIrqEdgeBit returned with error %d\n",retval);
    goto EXIT;
  }

  /********* enable IO00 interrupt locally **********/
  retval = PciedioSetIOIrqEnableBit(cardnum, 
	                                      0,   /*channel IO00*/
										  1);  /*endis = 1: enable the channel as an interrupt source*/
  if(retval!=0)
  { printf(">> PciedioSetIOIrqEnableBit returned with error %d\n",retval);
    goto EXIT;
  }

  /********* enable interrupts globally **********/
  retval = PciedioEnableGlobalIrq(cardnum);
  if(retval!=0)
  { printf(">> PciedioEnableGlobalIrq returned with error %d\n",retval);
    goto EXIT;
  }

  /********* start the FIFO **********/
  retval = PciedioStartFifo(cardnum);
  if(retval!=0)
  { printf(">> PciedioStartFifo returned with error %d\n",retval);
    goto EXIT;
  }

  /********** start the timer **********/
  if(PciedioStartTimer(cardnum))
   if(retval!=0)
  { printf(">> PciedioStartTimer returned with error %d\n",retval);
    goto EXIT;
  }

  

  /*wait on the interrupt service routine until a key has been pressed*/
  printf(">> Waiting on interrupts until a key has been pressed\n");
 
  while(!_kbhit())
  {	/*toggle the state of IO04 and write IO04: this will not result in an interrupt of IO04 as we havent't
	 enabled the input interrupt for channel IO04; we will see the toggling of IO04 in the FIFO buffer*/
	
	  /********** read present state of IO04 to varibale io04 **********/
	retval = PciedioGetIOBit(cardnum,
		                           4,    /*channel IO04*/
						&iochannel04);
    if(retval!=0)
 	 printf(">> PciedioGetIOBit returned with error %d\n",retval);

	/*toogle the state of IO04 and write IO04*/
	if(iochannel04==0)
	 iochannel04 = 1;
	else
	 iochannel04 = 0;
    retval = PciedioSetIOBit(cardnum,
		                           4,    /*channel IO04*/
					     iochannel04);   
    if(retval!=0)
 	 printf(">> PciedioSetIOBit returned with error %d\n",retval);

	delay_ms(5);  /*wait 5ms before the the next run starts*/
  }
  _getch();

  /********* stop the FIFO from collecting data **********/
  retval = PciedioStopFifo(cardnum);
  if(retval!=0)
  { printf(">> PciedioStopFifo returned with error %d\n",retval);
    goto EXIT;
  }

  /********** stop the timer **********/
  retval = PciedioStopTimer(cardnum);
  if(retval!=0)
  { printf(">> PciedioStopTimer returned with error %d\n",retval);
    goto EXIT;
  } 
 
  /********** disable the irq globally **********/
  retval = PciedioDisableGlobalIrq(cardnum);
  if(retval!=0)
  { printf(">> PciedioDisableGlobalIrq returned with error %d\n",retval);
    goto EXIT;
  } 

  /********* disable IO00 interrupt locally **********/
  retval = PciedioSetIOIrqEnableBit(cardnum, 
	                                      0,   /*channel IO00*/
										  0);  /*endis = 0: disable the channel as an interrupt source*/
  if(retval!=0)
  { printf(">> PciedioSetIOIrqEnableBit returned with error %d\n",retval);
    goto EXIT;
  }

  /********** disable the fifo irq locally **********/
  retval = PciedioDisableFifoIrq(cardnum);
  if(retval!=0)  
  { printf(">> PciedioDisableFifoIrq returned with error %d\n",retval);
    goto EXIT;
  } 

  /********** disable the timer irq locally **********/ 
  retval = PciedioDisableTimerIrq(cardnum);
  if(retval!=0)  
  { printf(">> PciedioDisableTimerIrq returned with error %d\n",retval);
    goto EXIT;
  } 

  /********** show the FIFO contents of the last run**********/
  printf("\nShow the last 100 FIFO entries:\n");
  for(i=0;i!=100;i++)
	printf("%03u: %08lxh\n",i,buffer[i]);


EXIT:
  /********** reset the card to its default state **********/
  retval = PciedioResetCard(cardnum);
  if(retval!=0)  
   printf(">> PciedioReset returned with error %d\n",retval);
  
  /********** release any resources allocated by PciedioOpenCards **********/ 
  retval = PciedioCloseCards();
  if(retval!=0)
   printf(">> PciedioCloseCards returned with error %d\n",retval);

  printf("\n\nHit any key to quit the program\n");
  
  _getch(); 

  return 0;
}


/**************************************************************************************************/
void delay_ms(WORD wait)
/**************************************************************************************************/
/*
Routine Description
    Perform a simple delay using the Windows API
*/
{ LARGE_INTEGER frequency; 
  LARGE_INTEGER t1, t2; 

  QueryPerformanceFrequency(&frequency);
 
  QueryPerformanceCounter(&t1);
  
  do
   QueryPerformanceCounter(&t2);
  while((1000.0*(t2.QuadPart-t1.QuadPart)/frequency.QuadPart) < wait);

  return;
}

