/*
Module     : QueryConfigurationData.cpp
Version    : 1.0   
Date       : 18.07.2016 
Project    : PCIeDIO sample
Environment: User mode
  
THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
PURPOSE.
*/

#include <stdio.h>
#include <conio.h>

#include "..\..\..\API\pciedio.h"

/**************************************************************************************************/
int main(void)
/**************************************************************************************************/
{ BYTE cardnum;
  BYTE indices;
  LONG retval;
  
  /********** open all the cards present in the system **********/
  retval = PciedioOpenCards();
  
  switch(retval)
  { case  0: printf(">> No PCIeDIO card present in the system\n");
             return 0;
             break;
    case  1: printf(">> 1 PCIeDIO card present in the system\n\n");
		     break;
	case  2:
	case  3:
	case  4:
	case  5:
	case  6:
	case  7:
	case  8:
		     printf(">> %d PCIeDIO cards present in the system\n\n",retval);
		 	 break;
	case -1:
	case -2: printf(">> PciedioOpenCards returned with error %d\n",retval);
		     return 0;
		 	 break;
    default: printf(">> PciedioOpenCards returned an impossible value %d\n", retval);
		     return 0;
			 break;
  }
  
  /********** determine the indices of the cards present in the system **********/
  PciedioGetIndices(&indices); 
  /*rem: no need to check the return value as we wouldn't be here when there was no card present*/
  
  /********** a simple printf of the configuration of each card which is present in the system *********/ 
  for(cardnum=0;cardnum!=8;cardnum++)                    
  { if(indices >> cardnum & 0x01)           /*is a card present at the index?*/  
    { BYTE  revpcb;                         /*used by PciedioGetRevisions*/        
      BYTE  revfpga;                        /*used by PciedioGetRevisions*/       
      BYTE  revkernelhigh, revkernellow;    /*used by PciedioGetRevisions*/       
      BYTE  revdllhigh, revdlllow;          /*used by PciedioGetRevisions*/        
	   
	  struct _PciedioConfig pciedioconfig;  /*used by PciedioGetPciConfiguration*/
	                                        /*struct _PciedioConfig is provided in pciedio.h*/
	  
	  DWORD address;                        /*used by PciedioGetAddressRange*/
	  DWORD range;                          /*used by PciedioGetAddressRange*/
	  BYTE  type;                           /*used by PciedioGetAddressRange*/ 

      BYTE  addressjumpers;                 /*used by PciedioGetPresentJumperSettings*/
      BYTE  userjumper;                     /*used by PciedioGetPresentJumperSettings*/


	  /********** receive the revisions of the components **********/
	  retval = PciedioGetRevisions(cardnum, 
		                          &revpcb, 
								  &revfpga, 
								  &revkernelhigh, 
								  &revkernellow, 
								  &revdllhigh, 
								  &revdlllow
								  );
      if(retval!=0)
       printf(">> PciedioGetRevisions returned with error %d\n",retval);
	  else
	  { printf(">> Revisions of the device at index %u:\n",cardnum);
        printf("   Revision of the PCB                : %u\n",revpcb);
        printf("   Revision of the FPGA               : %u\n",revfpga);
        printf("   Revision of the kernel-mode driver : %u.%u\n",revkernelhigh, revkernellow);
        printf("   Revision of the DLL                : %u.%u\n\n",revdllhigh, revdlllow);
	  }
   

      /********** receive the PCI configuration settings **********/ 
      retval = PciedioGetPciConfiguration(cardnum,
		                                 &pciedioconfig
										 );
	  if(retval!=0)
        printf(">> PciedioGetPciConfiguration returned with error %d\n",retval);
	  else
	  { printf(">> PCI configuration space of the device at index %u:\n",cardnum);
        printf("   Vendor ID                          : %xh\n",pciedioconfig.vendor);   
        printf("   Device ID                          : %xh\n",pciedioconfig.device);   
        printf("   Subsystem Vendor ID                : %xh\n",pciedioconfig.subvendor);   
        printf("   SubSystem ID                       : %xh\n",pciedioconfig.subsystem);   
        printf("   Revision ID                        : %xh\n",pciedioconfig.revision);  
        printf("   Base class Code                    : %xh\n",pciedioconfig.baseclass);
        printf("   Sub-class Code                     : %xh\n",pciedioconfig.subclass);     
        printf("   Header Type                        : %xh\n",pciedioconfig.headertype);     
        printf("   Base address                       : %08lxh\n",pciedioconfig.address);  	
        printf("   Address range                      : %xh bytes\n",pciedioconfig.range);
        printf("   Type of address space              : %u\n",pciedioconfig.type); 
        printf("   Interrupt Line                     : %xh (%u decimal)\n\n",pciedioconfig.interruptline,pciedioconfig.interruptline);  
	  }
 

	  /********** receive the address space and type without using a structure **********/ 
	  retval = PciedioGetAddressRange(cardnum,
	 	                             &address, 
								 	 &range, 
									 &type
									 );
      if(retval!=0)
        printf(">> PciedioGetAddressRange returned with error %d\n",retval);
	  else
	  { printf(">> PCI address range of the device at index %u:\n",cardnum);
        printf("   Base address                       : %08lxh\n",address);   
        printf("   Address range                      : %xh\n",range);  
        printf("   Type of address space              : %u\n\n",type);  
	  } 


	  /********** receive the jumper settings **********/ 
      retval = PciedioGetPresentJumperSettings(cardnum, &addressjumpers, &userjumper);
      if(retval!=0)
        printf(">> PciedioGetPresentJumperSettings returned with error %d\n",retval);
	  else
	  { printf(">> Jumper settings of the device at index %u:\n",cardnum);
        printf("   J0                                 : %u\n",addressjumpers & 0x01);
	    printf("   J1                                 : %u\n",(addressjumpers & 0x02) >> 1);
        printf("   J2                                 : %u\n",(addressjumpers & 0x04) >> 2);
		printf("   User                               : %u\n",userjumper);
		printf("\n\n");
	  }
    }
  }
 
  /********** release any resources which have been allocated by PciedioOpenCards **********/ 
  retval = PciedioCloseCards();
  if(retval!=0)
    printf(">> PciedioCloseCards returned with error %d\n",retval);

  printf("\n\nHit any key to quit the program\n");
  _getch(); 

  return 0;
}
