/*
Module     : Timer.cpp
Version    : 1.0   
Date       : 18.07.2016 
Project    : PCIeDIO sample
Environment: User mode
  
THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
PURPOSE.
*/

#include <stdio.h>
#include <conio.h>


#include "..\..\..\API\pciedio.h"

/**************************************************************************************************/
void __cdecl hIsr(BYTE cardnum, DWORD inputs, BYTE timer, BYTE fifo)
/**************************************************************************************************/
/*
Routine Description
    The interrupt service routine
*/
{ static DWORD count = 0;
  static DWORD error = 0;
  LONG   retval; 

  UNREFERENCED_PARAMETER(inputs);
  UNREFERENCED_PARAMETER(fifo);

  if(timer)
  { count++;

    if(count%1000==0)
     printf("card %u received %u timer interrupts with %u error(s)\n",cardnum, count, error);

    /********** set up the next run **********/
    retval = PciedioServiceTimerIrq(cardnum);
	if(retval!=0)
 	 error++;
  }
  else
   error++; /*we don't expect to count interrupts from the FIFO or the IOs as we haven't enabled them*/
  return;
}


/**************************************************************************************************/
int main(void)
/**************************************************************************************************/
{ BYTE  cardnum;
  BYTE  indices;
  LONG  retval;
  
  DWORD countval; /*used by PciedioGetTimerPresentState*/
  BYTE  start;    /*used by PciedioGetTimerPresentState*/
  BYTE  irqen;    /*used by PciedioGetTimerPresentState*/

  /********** open all the cards present in the system **********/
  retval = PciedioOpenCards();
  
  switch(retval)
  { case  0: printf(">> No PCIeDIO card present in the system\n");
             return 0;
             break;
    case  1: printf(">> 1 PCIeDIO card present in the system\n\n");
		     break;
	case  2:
	case  3:
	case  4:
	case  5:
	case  6:
	case  7:
	case  8:
		     printf(">> %d PCIeDIO cards present in the system\n\n",retval);
		 	 break;
	case -1:
	case -2: printf(">> PciedioOpenCards returned with error %d\n",retval);
		     return 0;
		 	 break;
    default: printf(">> PciedioOpenCards returned an impossible value %d\n", retval);
		     return 0;
			 break;
  }
  
  /********** determine the indices of the cards present in the system **********/
  PciedioGetIndices(&indices); 
  /*rem: no need to check the return value as we wouldn't be here when there was no card present*/
  
  /********** select the card with the lowest index present in the system *********/     
  for(cardnum=0;cardnum!=8;cardnum++)                    
  { if(indices >> cardnum & 0x01)           /*is a card present at the index?*/  
    { /*we have found the first valid index, so quit immediately*/           
      break;
    }
  }  
  
  printf(">> The card at index %u will be used to demonstrate the usage of the timer\n",cardnum);
 
  /********** get the present status of the timer **********/
  retval = PciedioGetTimerPresentState(cardnum, &countval, &start, &irqen);
  if(retval!=0)
  { printf(">> PciedioGetTimerPresentState returned with error %d\n",retval);
    goto EXIT;
  }
  else
   printf(">> Status of the timer of the card at index %u: %s, %s\n",cardnum, 
	                                                                 (start & 0x01) ? "started" :"stopped",
																	 (irqen & 0x01) ? "irq enabled" : "irg disabled"
																	);

  /********** install the isr **********/
  retval = PciedioOpenIrq(cardnum, &hIsr);
  if(retval!=0)
  { printf(">> PciedioOpenIrq returned with error %d\n",retval);
    goto EXIT;
  }

  /********* set up the timer interval to 1ms **********/
  retval = PciedioSetTimerInterval(cardnum,PCIEDIO_TIMER_1MS);
  if(retval!=0)
  { printf(">> PciedioSetTimerInterval returned with error %d\n",retval);
    goto EXIT;
  }

  /********* enable the timer as an interrupt source **********/
  retval = PciedioEnableTimerIrq(cardnum);
  if(retval!=0)
  { printf(">> PciedioEnableTimerIrq returned with error %d\n",retval);
    goto EXIT;
  }

  /********* enable interrupts globally **********/
  retval = PciedioEnableGlobalIrq(cardnum);
  if(retval!=0)
  { printf(">> PciedioEnableGlobalIrq returned with error %d\n",retval);
    goto EXIT;
  }

  /********** start the timer **********/
  if(PciedioStartTimer(cardnum))
   if(retval!=0)
  { printf(">> PciedioStartTimer returned with error %d\n",retval);
    goto EXIT;
  }

  /********** get the present status of the timer **********/
  retval = PciedioGetTimerPresentState(cardnum, &countval, &start, &irqen);
  if(retval!=0)
  { printf(">> PciedioGetTimerPresentState returned with error %d\n",retval);
    goto EXIT;
  }
  else
   printf(">> Status of the timer of the card at index %u: %s, %s\n",cardnum, 
	                                                                 (start & 0x01) ? "started" :"stopped",
																	 (irqen & 0x01) ? "irq enabled" : "irg disabled"
																	);

  /*wait on the interrupt service routine until a key has been pressed*/
  printf(">> Waiting on interrupts until a key has been pressed\n");
 
  while(!_kbhit())
   ;
  _getch();

  /********** stop the timer **********/
  retval = PciedioStopTimer(cardnum);
  if(retval!=0)
  { printf(">> PciedioStopTimer returned with error %d\n",retval);
    goto EXIT;
  } 
 
  /********** disable the irq globally **********/
  retval = PciedioDisableGlobalIrq(cardnum);
  if(retval!=0)
  { printf(">> PciedioDisableGlobalIrq returned with error %d\n",retval);
    goto EXIT;
  } 

  /********** disable the irq locally **********/ 
  retval = PciedioDisableTimerIrq(cardnum);
  if(retval!=0)  
  { printf(">> PciedioDisableTimerIrq returned with error %d\n",retval);
    goto EXIT;
  } 

  /********** get the present status of the timer **********/
  retval = PciedioGetTimerPresentState(cardnum, &countval, &start, &irqen);
  if(retval!=0)
  { printf(">> PciedioGetTimerPresentState returned with error %d\n",retval);
    goto EXIT;
  }
  else
   printf(">> Status of the timer of the card at index %u: %s, %s\n",cardnum, 
	                                                                 (start & 0x01) ? "started" :"stopped",
																	 (irqen & 0x01) ? "irq enabled" : "irg disabled"
																	);


EXIT:
  /********** reset the card to its default state **********/
  retval = PciedioResetCard(cardnum);
  if(retval!=0)  
   printf(">> PciedioReset returned with error %d\n",retval);
  
  /********** release any resources allocated by PciedioOpenCards **********/ 
  retval = PciedioCloseCards();
  if(retval!=0)
   printf(">> PciedioCloseCards returned with error %d\n",retval);

  printf("\n\nHit any key to quit the program\n");
  
  _getch(); 

  return 0;
}
