/*
Module     : Watchdog.cpp
Version    : 1.0   
Date       : 18.07.2016 
Project    : PCIeDIO sample
Environment: User mode
  
THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
PURPOSE.
*/

#include <stdio.h>
#include <conio.h>

#include "..\..\..\API\pciedio.h"

void delay_ms(WORD wait); /*perform a simple delay using the Windows API*/


/**************************************************************************************************/
int main(void)
/**************************************************************************************************/
{ BYTE  cardnum;
  BYTE  indices;
  LONG  retval;
  INT   c;
  DWORD inputs;
  BYTE  interval;
  BYTE  state;
 

  /********** open all the cards present in the system **********/
  retval = PciedioOpenCards();
  
  switch(retval)
  { case  0: printf(">> No PCIeDIO card present in the system\n");
             return 0;
             break;
    case  1: printf(">> 1 PCIeDIO card present in the system\n\n");
		     break;
	case  2:
	case  3:
	case  4:
	case  5:
	case  6:
	case  7:
	case  8:
		     printf(">> %d PCIeDIO cards present in the system\n\n",retval);
		 	 break;
	case -1:
	case -2: printf(">> PciedioOpenCards returned with error %d\n",retval);
		     return 0;
		 	 break;
    default: printf(">> PciedioOpenCards returned an impossible value %d\n", retval);
		     return 0;
			 break;
  }
  
  /********** determine the indices of the cards present in the system **********/
  PciedioGetIndices(&indices); 
  /*rem: no need to check the return value as we wouldn't be here when there was no card present*/
  
  /********** select the card with the lowest index present in the system *********/     
  for(cardnum=0;cardnum!=8;cardnum++)                    
  { if(indices >> cardnum & 0x01)           /*is a card present at the index?*/  
    { /*we have found the first valid index, so quit immediately*/           
      break;
    }
  }  
  
  printf(">> The card at index %u will be used to demonstrate the usage of the watchdog\n",cardnum);

  printf("\n>> WARNING!\n");
  printf("   Before running this application, disconnect all IOs to prevent any damage\n");
  printf("   as the application uses all IOs as outputs\n"); 
  printf("   Keep the external power connected to the card\n\n");
  printf(">> Press y to proceed or abort with q\n\n");
  
  c = 'a';
  while(c!='y')
  { if(_kbhit())
     { c = _getch();
       if(c=='q')
        goto EXIT;
     }
  }

   
  /********** get the present status of the watchdog **********/
  retval = PciedioGetWatchdogSettings(cardnum, &interval, &state);
  if(retval!=0)
  { printf(">>  PciedioGetWatchdogSettings returned with error %d\n",retval);
    goto EXIT;
  }
  else
   printf(">> Status of the watchdog of the card at index %u: interval=%u, %s\n",
                                    cardnum, interval, (state & 0x01) ? "running" :"disabled"
   );


  /********** set the output pattern 0xFFFFFFFF using DWORD access**********/
  retval = PciedioSetIODWord(cardnum, 0xFFFFFFFF); /*all IOs will be set to a high level*/
  if(retval!=0)  
   printf(">> PciedioSetIODWord returned with error %d\n",retval);
    
  /********** wait 1ms **********/
  delay_ms(1);

  /********** get the input pattern using DWORD access **********/
  retval = PciedioGetIODWord(cardnum, &inputs);
  if(retval!=0)  
   printf(">> PciedioGetIODWord returned with error %d\n",retval);
  else
  { if(inputs != 0xFFFFFFFF)
     printf(">> ERROR: input pattern %08lxh does not match the output pattern %08lxh\n", inputs, 0xFFFFFFFF);
  }
   
  /********** start the watchdog by setting up a valid interval **********/
  retval = PciedioSetWatchdogInterval(cardnum, PCIEDIO_WATCHDOG_100MS); /*PCIEDIO_WATCHDOG_100MS is defined in pciedio.h*/ 
  if(retval!=0)  
   printf(">> PciedioSetWatchdogInterval returned with error %d\n",retval);

  /********** get the present status of the watchdog again **********/
  retval = PciedioGetWatchdogSettings(cardnum, &interval, &state);
  if(retval!=0)
  { printf(">>  PciedioGetWatchdogSettings returned with error %d\n",retval);
    goto EXIT;
  }
  else
  { if(state!=0x01)
     printf(">> ERROR as the watchdog is not running\n");
    
    if(interval==0)
     printf(">> ERROR as the watchdog interval is 0\n");
    
    printf(">> Status of the watchdog of the card at index %u: interval=%u, %s\n",
                                    cardnum, interval, (state & 0x01) ? "running" :"disabled"
    );
  }

  printf(">> Service the watchdog until a key is hit\n");
  
  while(!_kbhit())
  { retval = PciedioSetIOUpdate(cardnum);  /*write the present setting again*/
    if(retval!=0)  
     printf(">> PciedioSetIOUpdate returned with error %d\n",retval);

	/********** check if the watchdog timer has elapsed **********/
	retval = PciedioGetWatchdogState(cardnum, &state);
    if(retval!=0)  
     printf(">> PciedioGetWatchdogState returned with error %d\n",retval);
	else
	{ if(state)
	   printf(">> ERROR as the watchdog timer has elapsed\n");
	}
  }
  _getch();

  printf(">> Stopped servicing the watchdog\n");

  printf(">> Wait for 200ms for the watchdog timer to have elapsed\n");

  delay_ms(200);

  /********** get the input pattern using DWORD access **********/
  retval = PciedioGetIODWord(cardnum, &inputs);
  if(retval!=0)  
   printf(">> PciedioGetIODWord returned with error %d\n",retval);
  else
  { if(inputs != 0x0)
      printf(">> ERROR: input pattern %08lxh does not match the dedicated output pattern %08lxh\n", inputs, 0x0);
    else
      printf(">> All IOs are at low level as planned\n");
  }

  /********** check if the watchdog timer has elapsed **********/
  retval = PciedioGetWatchdogState(cardnum, &state);
  if(retval!=0)  
    printf(">> PciedioGetWatchdogState returned with error %d\n",retval);
  else
  { if(state)
	 printf(">> The watchdog timer has elapsed as planned\n"); 
    else
	 printf(">> ERROR as the watchdog timer has not elapsed\n");
  }

EXIT:
  /********** reset the card to its default state **********/
  retval = PciedioResetCard(cardnum);
  if(retval!=0)  
   printf(">> PciedioReset returned with error %d\n",retval);
  
  /********** release any resources allocated by PciedioOpenCards **********/ 
  retval = PciedioCloseCards();
  if(retval!=0)
   printf(">> PciedioCloseCards returned with error %d\n",retval);

  printf("\n\nHit any key to quit the program\n");
  
  _getch(); 

  return 0;
}


/**************************************************************************************************/
void delay_ms(WORD wait)
/**************************************************************************************************/
/*
Routine Description
    Perform a simple delay using the Windows API
*/
{ LARGE_INTEGER frequency; 
  LARGE_INTEGER t1, t2; 

  QueryPerformanceFrequency(&frequency);
 
  QueryPerformanceCounter(&t1);
  
  do
   QueryPerformanceCounter(&t2);
  while((1000.0*(t2.QuadPart-t1.QuadPart)/frequency.QuadPart) < wait);

  return;
}